#![no_std]
#![no_main]
#![feature(panic_info_message)]
#![feature(abi_avr_interrupt)]
#![feature(asm_experimental_arch)]
#![feature(thread_local)]
#![allow(clippy::wrong_self_convention)]

#[macro_use]
extern crate static_pins;

use arduino_hal::{
    delay_ms,
    hal::{
        pins,
        port::{PD2, PD3, PD5},
    },
    port::{mode::Output, Pin},
    Peripherals,
};

mod half_duplex_serial;
mod keyboard;
mod panic_handler;
mod static_pins_impl;

use avr_device::interrupt::{enable, free};
use half_duplex_serial::{HalfDuplexSerial, PollError, SoftSerial};
use keyboard::*;

#[allow(unused)]
use panic_handler::*;

pub static mut SERIAL: Option<HalfDuplexSerial<PD3>> = None;

static mut RX_LED: Option<Pin<Output, PD5>> = None;
static mut POLL_INDICATOR: Option<Pin<Output, PD2>> = None;

fn error_poll_status_display(delay: u16) {
    unsafe {
        if let Some(poll_status) = POLL_INDICATOR.as_mut() {
            for _ in 0..=5 {
                delay_ms(delay);
                poll_status.toggle();
            }
            poll_status.set_low();
        }
        delay_ms(500)
    }
}

#[arduino_hal::entry]
fn main() -> ! {
    let dp = Peripherals::take().unwrap();

    let pins = pins!(dp);

    let mut matrix = KeyboardMatrix::new(
        pins_array!("cols", pins, pd0, pd4),
        pins_array!("rows", pins, pd7),
    );

    let mut rx_led = pins.pd5.into_output();
    let poll_indicator = pins.pd2.into_output();
    // let one_wire_pin = StaticPin::new(pins.pd3.into_pull_up_input());
    pins.pb0.into_output().set_low();
    rx_led.set_low();

    unsafe {
        SERIAL = Some(HalfDuplexSerial::new(
            pins.pd3.into_pull_up_input(),
            &dp.EXINT,
        ));
        POLL_INDICATOR = Some(poll_indicator);
        RX_LED = Some(rx_led);
        enable();

        loop {
            let pressed_keys = matrix.scan();
            if pressed_keys[0].is_some() || pressed_keys[1].is_some() {
                free(|_| {
                    if let Some(serial) = SERIAL.as_ref() {
                        match serial.poll() {
                            Ok(_) => {
                                if let Some(led) = POLL_INDICATOR.as_mut() {
                                    led.set_high();
                                }
                            }
                            Err(PollError::NotFound) => error_poll_status_display(150),
                            Err(PollError::NotReady) => error_poll_status_display(750),
                        }
                    }
                });
            }
        }
    }
}

#[avr_device::interrupt(atmega32u4)]
unsafe fn INT3() {
    if let Some(serial) = SERIAL.as_ref() {
        serial.response();
        if let Some(led) = POLL_INDICATOR.as_mut() {
            led.toggle();
        }
    }
}
